//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     scope.cxx
// Purpose:    Set of all variables in the current formula
// Last Change:04.04.2002
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2000-2002 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"
#include "out.h"
#include "scope.h"

//=============================================================================
// Class Variables:
//=============================================================================

//-----------------------------------------------------------------------------
// Vars: Table with pointers to variables.
//-----------------------------------------------------------------------------

var_t	scope_c::Vars[VER_MAX_VARS];

//-----------------------------------------------------------------------------
// NumVars: Number of variables in table.
//-----------------------------------------------------------------------------

int	scope_c::NumVars = 0;

//-----------------------------------------------------------------------------
// NextVar: Index of next variable in the loop/scan over all variables.
//-----------------------------------------------------------------------------

int	scope_c::NextVar = 0;

//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
bool_t	scope_c::valid(void)
{
	// NumVars must be reasonable:
	CHECK(NumVars >= 0, "scope_c::valid: NumVars is negative");
	CHECK(NumVars <= VER_MAX_VARS, "scope_c::valid: NumVars too large");

	// NextVar must be reasonable:
	CHECK(NextVar >= 0, "scope_c::valid: NextVar is negative");
	CHECK(NextVar <= NumVars, "scope_c::valid: NextVar too large");

	// Ok:
	return(BOOL_TRUE);
}
#endif

//-----------------------------------------------------------------------------
// Find or Construct a Variable:
//-----------------------------------------------------------------------------

var_t scope_c::find(str_t name_ptr, str_t name_end)
{
	int	i;
	var_t*	var_p;
	register str_t p;
	register str_t q;

	// Check internal consistency:
	CHECK(valid(), "scope_c::find: Internal inconsistency");

	// Check input parameters:
	CHECK(name_ptr != 0, "scope_c::find: name_ptr is NIL");
	CHECK(name_end != 0, "scope_c::find: name_end is NIL");
	CHECK(name_end > name_ptr,
		"scope_c::find: name_end before name_ptr");

	// Do a linear search:
	var_p = Vars;
	for(i = NumVars; --i >= 0; ) {

		// Compare Variable Names:
		p = (*var_p)->name();
		q = name_ptr;
		while(*p && q!=name_end && *p==*q) {
			p++;
			q++;
		}

		// Return variable if names match.
		if(*p == '\0' && q == name_end)
			return(*var_p);

		// Otherwise we look at the next variable:
		var_p++;
	}

	// Not found. So we need to construct a new variable:
	if(NumVars == VER_MAX_VARS)
		return(0);
	*var_p = new var_c(name_ptr, name_end);
	NumVars++;

	// Return the new variable:
	return(*var_p);
}

//-----------------------------------------------------------------------------
// Print This Variable List:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
void scope_c::print(void)
{
	int i;
	var_t *var_p;

	// Check internal consistency:
	CHECK(valid(), "scope_c::print: Internal inconsistency");

	// Start output:
	out_c::print_str("SCOPE: ");

	// Handle empty list:
	if(NumVars == 0)
		out_c::print_str("<EMPTY>");

	// Handle non-empty list:
	else {
		var_p = Vars;
		for(i = NumVars; --i >=0; ) {
			(*var_p)->print();
			if(i != 0)
				out_c::print_str(", ");
		}
	}

	// Finish output:
	out_c::print_char('.');
}
#endif

